#if !defined( __EMS)
#define __EMS
//******************************************************
// Please read EMS.DOC for more details and examples 
//******************************************************

#define  UNMAP    0xFFFF   //used to unmap logical ems pages.
#define  EMSPAGE  16384    //16kb

typedef unsigned size_t;
typedef unsigned long Ulong;

typedef struct EMS_Status{
      int UsedHandles;
      int UnusedPages;
      int EMS_StatusCode;
      }EMS_Status;

typedef struct EMS_Hn_Pg{
	int Handle;
	int Pages;
     }EMS_Hn_Pg;

typedef struct Address{
     unsigned int Off;
     unsigned int Segg;
     }Address ;


typedef EMS_Hn_Pg          EMS_PageArray[255];
#if !defined( __EMS_FUNC_)
#define __EMS_FUNC_

//******** global variables below are located in ems.cpp ******************

//**************************************************************************
// set to pages that ems virtual memory manager will use. if USE_EMSPAGES is
// set to 0 then all available ems pages will be allocated by the virtual
// memory manager.
extern  int                USE_EMSPAGES;
//**************************************************************************

extern  int          HAS_EMS;           // true if ems memory is on computer
extern  int          HAS_UPPERMEM;      // true if memory >640 k is availale
extern  int          HAS_VIRTUAL_EMS_PTRS; //true em_pointers is available
extern  int          MAXEMSPAGES;       // contains total ems pages available
extern  unsigned     EMS_SEG;           // ems page frame segment
extern  void far *   EMSBASEADDRESS;    // ems page frame address
extern  EMS_Status   E_M_S_Status;      // struct to retrieve ems status
#endif

//*************************************************************************
void * operator new    ( size_t size );       // overloaded new operator.
void   operator delete ( void * ptr  );       // overloaded delete operator.
// new and delete tries to use upper memory if its avalable otherwise it
// uses memory from the far heap.
// can allocat conventional mem above 640k and <1024k
//*************************************************************************


/************************ POSIBLE EMS ERRORS ******************************
  You can test  the error value against these values to see exactly what
  kind of error has occured.

     0x00   =  No error
     0x80   =  EMS software error
     0x81   =  Hardware error
     0x84   =  Function code undefined
     0x85   =  No EMS handles Left
     0x87   =  Insufficient pages
     0x88   =  Insufficient unallocated pages
     0x89   =  Cannot allocate 0 pages
     0x90   =  Low conventional memory, cannot allocate buffer space
***************************************************************************/


//***************************************************************
//  This function allocates expanded memory pages from the
//   memory manager. You can use these pages to store information
void far getems_mem(int /*Pages*/, int * /**Handle*/, int * /**error*/);


//***************************************************************
// This function fills an array of EMS_PageArray type with
// all the open handles and the amount of pages assigned to
// these pages.
void far used_pages(EMS_PageArray &/*EMSarray*/,int */*UsedHandles*/,int */*error*/ );

//***************************************************************
// Maps a logical page to a physical page. Pass 0xFFFF to unmap
// Physical page
void far map_un_mapems(int /*Handle*/, unsigned int /*LogPage*/, char /*PhyPage*/, int */*error*/);

//***************************************************************
// Returns the number of pages associated with a EMS handle
int far pagewithhandle(int /*Handle*/, int */*error*/);


//***************************************************************
// EmsDataPtr returns a pointer that points to data  in the
// Physical page in the EMS page frame. pointer = (void far *)
void far * emsdataptr(int /*PhyPage*/,int /*Offset*/);


//***************************************************************
// returns the current ems memory status in a structure of EMS_Status
void far currentemsstatus(EMS_Status */*EMMStatus */,int */*error*/);

/***********************************************************************/
//	Frees all EMS pages associated with a ems handle
void far freehandle(int /*Handle*/, int  */*error*/ );


/***************************************************************************/
#if !defined( _em_CLASS)

class em_pointer{
 int      handle;
 friend  void far em_free(em_pointer e);        // free ems bytes on ems heap
 friend  em_pointer far em_malloc(long nbytes); // alocate nbytes on ems heap

 friend  int  far em_memset(em_pointer e, int c, long nbytes);
 friend  int  far em_copyfrom(void huge* dest, const em_pointer src, long nbytes);
 friend  int  far em_copyto  (em_pointer dest, const void huge* src, long nbytes);
 friend  int  far em_memmove (em_pointer dest, const em_pointer src, long nbytes);

 friend  em_pointer far operator+=(em_pointer e,long i);  // pointer offset incrementation
 friend  em_pointer far operator-=(em_pointer e,long i);  // pointer offset decrementation
 friend  em_pointer far operator--(em_pointer e);         // pointer offset decrementation
 friend  em_pointer far operator++(em_pointer e);         // pointer offset incrementation

 friend  int  far operator! (em_pointer e);               // NULL comparison
 friend  int  far operator==(em_pointer e1,long l);       // NULL comparison
 friend  int  far operator!=(em_pointer e1,long l);       // NULL comparison
 friend  int  far operator==(em_pointer e1,em_pointer e2);// pointer to pointer comparison
 friend  int  far operator!=(em_pointer e1,em_pointer e2);// pointer to pointer comparison

 public:
	 void operator= (em_pointer e);               // pointer to pointer assignment
	 void operator= (long);                       // NULL assignment
	 em_pointer(long  l=NULL);                    // NULL initalization
};


//***********************************************************************/
// memset sets the first nbytes of the e to the character c.
int far em_memset(em_pointer e, int c, long nbytes);

/***********************************************************************/
// copies pages of ems memory from an em_pointer  to another em_pointer.
// the destination buffer must be large enough to hold the requested pages.
int  far em_memmove(em_pointer dest, const em_pointer src, long nbytes);

/***********************************************************************/
// copies pages of ems memory from an em_pointer  to conventional memory.
// destination buffer must be large enough to hold the requested pages.
int far  em_copyfrom( void far* dest, const em_pointer src, long nbytes);


/***********************************************************************/
// copies pages of conventional memory to an em_pointer that has
// allocated enough ems pages to hold the request.
// destination buffer must be large enough to hold the requested pages.
int far  em_copyto(em_pointer dest, const void far* src, long nbytes);


/***********************************************************************/
// allocate certain pages of ems memory and return it in an em_pointer.
// if not enough pages is available then the em_pointer returned is = NULL.
// em_pointers can be tested for null condition.
em_pointer far em_malloc(int pages);


/***********************************************************************/
//free deallocates memory pages allocated by a previous call to em_malloc
void far em_free(em_pointer e);

// returns the size of the largest contigious block of ems memory that
// can be allocated.  If you want a larger block and there is more ems
// memory available, the virtual pointer memory manager will try to
// defragment the memory.
long far ems_maxavail();


// returns the amount  of memory remaining on the ems heap that
// can be allocated with em_malloc.
long far ems_memavail();


#endif
#endif